==> rtracklayer_1.8.1.tar.gz: rtracklayer/R/bigWig.R <==
### UCSC bigWig format 

.allowedColNames <- list(bigWig = "score")

.validateColNames <- function(object, format) {
  allowedColNames <- .allowedColNames[[format]]
  invalidColNames <- setdiff(colnames(object), allowedColNames)
  if (length(invalidColNames))
    paste("Column names",
          paste("'", invalidColNames, "'", sep = "", collapse=", "),
          "are invalid for the", format, "format.")
  else NULL
}

setClass("BigWigSelection", prototype = prototype(colnames = "score"),
         contains = "RangedSelection")

setValidity("BigWigSelection",
            function(object) {
              .validateColNames(object, "bigWig")
            })

BigWigSelection <- function(ranges = RangesList(), colnames = "score") {
  if (!is.character(colnames) ||
      (length(colnames) && !identical(colnames, "score")))
    stop("'score' is the only valid column for BigWig")
  if (!is(ranges, "RangesList")) 
    stop("'ranges' must be a RangesList")
  new("BigWigSelection", ranges = ranges, colnames = colnames)
}

setAs("RangesList", "BigWigSelection", function(from) {
  new("BigWigSelection", as(from, "RangedSelection"))
})

setGeneric("export.bw",
           function(object, con,
                    dataFormat = c("auto", "variableStep", "fixedStep",
                      "bedGraph"),
                    seqlengths = NULL, compress = TRUE, ...)
           standardGeneric("export.bw"))

setMethod("export.bw", "ANY",
          function(object, con,
                   dataFormat = c("auto", "variableStep", "fixedStep",
                     "bedGraph"),
                   seqlengths, compress, genome, ...)
          {
            rd <- as(object, "RangedData")
            if (!is.null(genome))
              genome(rd) <- genome
            export.bw(rd, con, dataFormat, seqlengths, compress, ...)
          })

setMethod("export.bw", c("RangedData", "character"),
          function(object, con,
                   dataFormat = c("auto", "variableStep", "fixedStep",
                                  "bedGraph"),
                   seqlengths, compress)
          {
            score <- score(object)
            if (!is.numeric(score) || any(is.na(score)))
              stop("The score must be numeric, without any NA's")
            if (!IRanges:::isTRUEorFALSE(compress))
              stop("'compress' must be TRUE or FALSE")
            if (is.null(seqlengths) && !is.null(genome(object)))
              seqlengths <- seqlengths(.genomeForID(genome(object)))
            if (is.null(seqlengths))
              stop("Unable to determine seqlengths; either specify ",
                   "'seqlengths' or specify a genome on 'object' that ",
                   "is known to BSgenome")
            if (!is.numeric(seqlengths) ||
                !all(names(object) %in% names(seqlengths)))
              stop("seqlengths must be numeric and indicate a length for ",
                   "each sequence in 'object'")
            sectionPtr <- NULL # keep adding to the same linked list
            .bigWigWriter <- function(chromData, con, dataFormat) {
              sectionPtr <<- .Call(BWGSectionList_add, sectionPtr,
                                   names(chromData)[1],
                                   as(ranges(chromData)[[1]], "IRanges"),
                                   as.numeric(score(chromData)), dataFormat)
            }
            dataFormat <- match.arg(dataFormat)
            if (dataFormat == "auto")
              format <- chooseGraphType(object)
            else format <- dataFormat
            on.exit(.Call(BWGSectionList_cleanup, sectionPtr))
            if (format == "bedGraph")
              lapply(object, .bigWigWriter, con, dataFormat)
            else export.wigLines(object, con, dataFormat, .bigWigWriter)
            storage.mode(seqlengths) <- "integer"
            .Call(BWGSectionList_write, sectionPtr, seqlengths,
                  compress, con)
          })

setGeneric("import.bw",
           function(con, selection = BigWigSelection(...), ...)
           standardGeneric("import.bw"))


setMethod("import.bw", "character",
          function(con, selection = BigWigSelection(...), ...)
          {
            if (!IRanges:::isSingleString(con))
              stop("'con' must be a single string, specifying a path")
            selection <- as(selection, "BigWigSelection")
            validObject(selection)
            normRanges <- as(ranges(selection), "NormalIRangesList")
            rd <- .Call(BWGFile_query, con, as.list(normRanges),
                        identical(colnames(selection), "score"))
            ## Unfortunately, the bigWig query API is such that we can
            ## end up with multiple hits.
            if (any(width(rd) > 2)) {
              hits <- queryHits(findOverlaps(ranges(rd), normRanges))
              rd <- rd[!duplicated(hits),]
            }
            rd
          })

==> rtracklayer_1.12.5.tar.gz: rtracklayer/R/bigWig.R <==
### UCSC bigWig format 

.allowedColNames <- list(bigWig = "score")

.validateColNames <- function(object, format) {
  allowedColNames <- .allowedColNames[[format]]
  invalidColNames <- setdiff(colnames(object), allowedColNames)
  if (length(invalidColNames))
    paste("Column names",
          paste("'", invalidColNames, "'", sep = "", collapse=", "),
          "are invalid for the", format, "format.")
  else NULL
}

setClass("BigWigSelection", prototype = prototype(colnames = "score"),
         contains = "RangedSelection")

setValidity("BigWigSelection",
            function(object) {
              .validateColNames(object, "bigWig")
            })

BigWigSelection <- function(ranges = GRanges(), colnames = "score") {
  if (!is.character(colnames) ||
      (length(colnames) && !identical(colnames, "score")))
    stop("'score' is the only valid column for BigWig")
  if (is.character(ranges))
    new("BigWigSelection", GenomicSelection(ranges, colnames = colnames))
  else {
    new("BigWigSelection", ranges = as(ranges, "RangesList"),
        colnames = colnames)
  }
}

setAs("RangesList", "BigWigSelection", function(from) {
  new("BigWigSelection", as(from, "RangedSelection"))
})

setAs("GRanges", "BigWigSelection", function(from) {
  as(as(from, "RangesList"), "BigWigSelection")
})

setGeneric("export.bw",
           function(object, con,
                    dataFormat = c("auto", "variableStep", "fixedStep",
                      "bedGraph"),
                    seqlengths = NULL, compress = TRUE, ...)
           standardGeneric("export.bw"))

setMethod("export.bw", "ANY",
          function(object, con,
                   dataFormat = c("auto", "variableStep", "fixedStep",
                     "bedGraph"),
                   seqlengths, compress, genome, ...)
          {
            rd <- as(object, "RangedData")
            if (!is.null(genome))
              genome(rd) <- genome
            export.bw(rd, con, dataFormat, seqlengths, compress, ...)
          })

setMethod("export.bw", c("RangedData", "character"),
          function(object, con,
                   dataFormat = c("auto", "variableStep", "fixedStep",
                                  "bedGraph"),
                   seqlengths, compress)
          {
            score <- score(object)
            if (!is.numeric(score) || any(is.na(score)))
              stop("The score must be numeric, without any NA's")
            if (!IRanges:::isTRUEorFALSE(compress))
              stop("'compress' must be TRUE or FALSE")
            if (is.null(seqlengths) && !is.null(genome(object)))
              seqlengths <- seqlengths(.genomeForID(genome(object)))
            if (is.null(seqlengths))
              stop("Unable to determine seqlengths; either specify ",
                   "'seqlengths' or specify a genome on 'object' that ",
                   "is known to BSgenome")
            if (!is.numeric(seqlengths) ||
                !all(names(object) %in% names(seqlengths)))
              stop("seqlengths must be numeric and indicate a length for ",
                   "each sequence in 'object'")
            sectionPtr <- NULL # keep adding to the same linked list
            .bigWigWriter <- function(chromData, con, dataFormat) {
              sectionPtr <<- .Call(BWGSectionList_add, sectionPtr,
                                   names(chromData)[1],
                                   as(ranges(chromData)[[1]], "IRanges"),
                                   as.numeric(score(chromData)), dataFormat)
            }
            dataFormat <- match.arg(dataFormat)
            if (dataFormat == "auto")
              format <- chooseGraphType(object)
            else format <- dataFormat
            on.exit(.Call(BWGSectionList_cleanup, sectionPtr))
            if (format == "bedGraph")
              lapply(object, .bigWigWriter, con, dataFormat)
            else export.wigLines(object, con, dataFormat, .bigWigWriter)
            storage.mode(seqlengths) <- "integer"
            .Call(BWGSectionList_write, sectionPtr, seqlengths,
                  compress, con)
          })

setGeneric("import.bw",
           function(con, selection = BigWigSelection(...), ...)
           standardGeneric("import.bw"))


setMethod("import.bw", "character",
          function(con, selection = BigWigSelection(...), ...)
          {
            if (!IRanges:::isSingleString(con))
              stop("'con' must be a single string, specifying a path")
            selection <- as(selection, "BigWigSelection")
            validObject(selection)
            normRanges <- as(ranges(selection), "NormalIRangesList")
            rd <- .Call(BWGFile_query, con, as.list(normRanges),
                        identical(colnames(selection), "score"))
            ## Unfortunately, the bigWig query API is such that we can
            ## end up with multiple hits.
            if (any(width(rd) > 2)) {
              hits <- queryHits(findOverlaps(ranges(rd), normRanges))
              rd <- rd[!duplicated(hits),]
            }
            rd
          })
